<?php

namespace Drupal\pf_email\Plugin\PushFrameworkChannel;

use Drupal;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Utility\Token;
use Drupal\push_framework\ChannelBase;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the push framework channel.
 *
 * @ChannelPlugin(
 *   id = "email",
 *   label = @Translation("Email"),
 *   description = @Translation("Provides the email channel plugin.")
 * )
 */
class Email extends ChannelBase {

  /**
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * @inheritDoc
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, LoggerChannelInterface $logger, RendererInterface $renderer, Token $token, MailManagerInterface $mail_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $config_factory, $entity_type_manager, $logger, $renderer, $token);
    $this->mailManager = $mail_manager;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('logger.channel.push_framework'),
      $container->get('renderer'),
      $container->get('token'),
      $container->get('plugin.manager.mail')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigName(): string {
    return 'pf_email.settings';
  }

  /**
   * {@inheritdoc}
   */
  public function applicable(UserInterface $user): bool {
    return $this->active;
  }

  /**
   * {@inheritdoc}
   */
  public function send(UserInterface $user, ContentEntityInterface $entity, array $content, int $attempt): string {
    $language = $user->getPreferredLangcode();
    if (!isset($content[$language])) {
      $language = array_keys($content)[0];
    }
    $message = $this->mailManager->mail('pf_email', 'notification', $user->getEmail(), $user->getPreferredLangcode(), [
      'subject' => $content[$language]['subject'],
      'body' => $content[$language]['body'],
      'is html' => $content[$language]['is html'],
    ]);

    return $message['result'] ? self::RESULT_STATUS_SUCCESS : self::RESULT_STATUS_FAILED;
  }

}
